@@ -10,6 +10,13 @@ from awkward.types import ArrayType, ListType, NumpyType, OptionType, RegularTyp
 to_list = ak.operations.to_list
 
 
+@pytest.fixture(scope="function", autouse=True)
+def cleanup_cuda():
+    yield
+    cp._default_memory_pool.free_all_blocks()
+    cp.cuda.Device().synchronize()
+
+
 def test_0184_concatenate_number():
     a1 = ak.highlevel.Array([[1, 2, 3], [], [4, 5]]).layout
     a2 = ak.highlevel.Array([[[1.1], [2.2, 3.3]], [[]], [[4.4], [5.5]]]).layout
@@ -875,3 +882,199 @@ def test_2663_broadcast_tuples_tuple_tuple():
             ]
         ],
     )
+
+
+def test_flatten_UnionArray():
+    content1 = ak.operations.from_iter(
+        [[1.1], [2.2, 2.2], [3.3, 3.3, 3.3]], highlevel=False
+    )
+    content2 = ak.operations.from_iter(
+        [[[3, 3, 3], [3, 3, 3], [3, 3, 3]], [[2, 2], [2, 2]], [[1]]], highlevel=False
+    )
+    content3 = ak.operations.from_iter(
+        [
+            [["3", "3", "3"], ["3", "3", "3"], ["3", "3", "3"]],
+            [["2", "2"], ["2", "2"]],
+            [["1"]],
+        ],
+        highlevel=False,
+    )
+    tags = ak.index.Index8(np.array([0, 1, 0, 1, 0, 1], dtype=np.int8))
+    index = ak.index.Index64(np.array([0, 0, 1, 1, 2, 2], dtype=np.int64))
+    array = ak.contents.UnionArray(tags, index, [content1, content2])
+
+    assert to_list(array) == [
+        [1.1],
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [2.2, 2.2],
+        [[2, 2], [2, 2]],
+        [3.3, 3.3, 3.3],
+        [[1]],
+    ]
+    assert to_list(array[1:]) == [
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [2.2, 2.2],
+        [[2, 2], [2, 2]],
+        [3.3, 3.3, 3.3],
+        [[1]],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(array)) == [
+        1.1,
+        [3, 3, 3],
+        [3, 3, 3],
+        [3, 3, 3],
+        2.2,
+        2.2,
+        [2, 2],
+        [2, 2],
+        3.3,
+        3.3,
+        3.3,
+        [1],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(array[1:])) == [
+        [3, 3, 3],
+        [3, 3, 3],
+        [3, 3, 3],
+        2.2,
+        2.2,
+        [2, 2],
+        [2, 2],
+        3.3,
+        3.3,
+        3.3,
+        [1],
+    ]
+
+    array = ak.contents.UnionArray(tags, index, [content2, content3])
+
+    assert to_list(array) == [
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [["3", "3", "3"], ["3", "3", "3"], ["3", "3", "3"]],
+        [[2, 2], [2, 2]],
+        [["2", "2"], ["2", "2"]],
+        [[1]],
+        [["1"]],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(array, axis=2)) == [
+        [3, 3, 3, 3, 3, 3, 3, 3, 3],
+        ["3", "3", "3", "3", "3", "3", "3", "3", "3"],
+        [2, 2, 2, 2],
+        ["2", "2", "2", "2"],
+        [1],
+        ["1"],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(array[1:], axis=2)) == [
+        ["3", "3", "3", "3", "3", "3", "3", "3", "3"],
+        [2, 2, 2, 2],
+        ["2", "2", "2", "2"],
+        [1],
+        ["1"],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(array[:, 1:], axis=2)) == [
+        [3, 3, 3, 3, 3, 3],
+        ["3", "3", "3", "3", "3", "3"],
+        [2, 2],
+        ["2", "2"],
+        [],
+        [],
+    ]
+
+
+def test_0150_flatten_UnionArray():
+    content1 = ak.operations.from_iter(
+        [[1.1], [2.2, 2.2], [3.3, 3.3, 3.3]], highlevel=False
+    )
+    content2 = ak.operations.from_iter(
+        [[[3, 3, 3], [3, 3, 3], [3, 3, 3]], [[2, 2], [2, 2]], [[1]]], highlevel=False
+    )
+    content3 = ak.operations.from_iter(
+        [
+            [["3", "3", "3"], ["3", "3", "3"], ["3", "3", "3"]],
+            [["2", "2"], ["2", "2"]],
+            [["1"]],
+        ],
+        highlevel=False,
+    )
+    tags = ak.index.Index8(np.array([0, 1, 0, 1, 0, 1], dtype=np.int8))
+    index = ak.index.Index64(np.array([0, 0, 1, 1, 2, 2], dtype=np.int64))
+    array = ak.contents.UnionArray(tags, index, [content1, content2])
+    cuda_array = ak.to_backend(array, "cuda")
+
+    assert to_list(cuda_array) == [
+        [1.1],
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [2.2, 2.2],
+        [[2, 2], [2, 2]],
+        [3.3, 3.3, 3.3],
+        [[1]],
+    ]
+    assert to_list(cuda_array[1:]) == [
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [2.2, 2.2],
+        [[2, 2], [2, 2]],
+        [3.3, 3.3, 3.3],
+        [[1]],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(cuda_array)) == [
+        1.1,
+        [3, 3, 3],
+        [3, 3, 3],
+        [3, 3, 3],
+        2.2,
+        2.2,
+        [2, 2],
+        [2, 2],
+        3.3,
+        3.3,
+        3.3,
+        [1],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(cuda_array[1:])) == [
+        [3, 3, 3],
+        [3, 3, 3],
+        [3, 3, 3],
+        2.2,
+        2.2,
+        [2, 2],
+        [2, 2],
+        3.3,
+        3.3,
+        3.3,
+        [1],
+    ]
+
+    array = ak.contents.UnionArray(tags, index, [content2, content3])
+    cuda_array = ak.to_backend(array, "cuda")
+
+    assert to_list(cuda_array) == [
+        [[3, 3, 3], [3, 3, 3], [3, 3, 3]],
+        [["3", "3", "3"], ["3", "3", "3"], ["3", "3", "3"]],
+        [[2, 2], [2, 2]],
+        [["2", "2"], ["2", "2"]],
+        [[1]],
+        [["1"]],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(cuda_array, axis=2)) == [
+        [3, 3, 3, 3, 3, 3, 3, 3, 3],
+        ["3", "3", "3", "3", "3", "3", "3", "3", "3"],
+        [2, 2, 2, 2],
+        ["2", "2", "2", "2"],
+        [1],
+        ["1"],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(cuda_array[1:], axis=2)) == [
+        ["3", "3", "3", "3", "3", "3", "3", "3", "3"],
+        [2, 2, 2, 2],
+        ["2", "2", "2", "2"],
+        [1],
+        ["1"],
+    ]
+    assert ak.operations.to_list(ak.operations.flatten(cuda_array[:, 1:], axis=2)) == [
+        [3, 3, 3, 3, 3, 3],
+        ["3", "3", "3", "3", "3", "3"],
+        [2, 2],
+        ["2", "2"],
+        [],
+        [],
+    ]
